/* -*- Mode: java; tab-width: 8; c-basic-offset: 4 -*-
 * Copyright  1997, 1998 Netscape Communications Corporation, All Rights Reserved.
 */

package com.netscape.javascript.CORBA;

import com.netscape.javascript.*;
import org.omg.CORBA.TCKind;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.Any;
import org.omg.CORBA.portable.*;
import org.omg.CORBA.CompletionStatus;

/**
 * This class reflects primitive IDL objects into the JS environment.
 *
 * It also can be used in the future to implement CORBA.Short (value) type of objects
 * for the IRless case
 *
 * @author Ruslan Belkin
 */

public class   Simple   extends ScriptableObject {

    public Simple (Any any)
    {
        this.any = any;

        try {
            decode ();
        }
        catch (org.omg.CORBA.SystemException se)
        {
            Init.raise (new SystemException (se));
        }
    }

    public Simple (TypeCode tc)
    {
        any = Init.orb.create_any ();
        any.type (tc);

        OutputStream os = any.create_output_stream ();

        try {
            Marshal.copyObject (null, os, tc);
            any.read_value (os.create_input_stream (), tc);
            decode ();
        }
        catch (org.omg.CORBA.SystemException se)
        {
            Init.raise (new SystemException (se));
        }
    }

    // this is for the IR-less case
    //
    public Simple (TypeCode tc, Object obj)
    {
        any = Init.orb.create_any ();
        any.type (tc);

        OutputStream os = any.create_output_stream ();

        try {
            writeObject (os, tc, obj);
            any.read_value (os.create_input_stream (), tc);
            decode ();
        }
        catch (org.omg.CORBA.SystemException se)
        {
            Init.raise (new SystemException (se));
        }
    }

    public String getClassName ()
    {
        return "Simple";
    }

    public Object getDefaultValue (Class hint)
    {
 	    if (hint == null || hint == String.class)
       	    return toString ();

    	return valueOf ();
    }

    public Object valueOf ()
    {
        return obj;

    }

    public String toString ()
    {
        return (String)obj;
    }

    public static Object extractObject (InputStream is, TypeCode tc)
        throws org.omg.CORBA.MARSHAL
    {
        switch (tc.kind().value ())
        {
            case TCKind._tk_null:
            case TCKind._tk_void:
                return null;
            case TCKind._tk_boolean:
                return new Boolean (is.read_boolean ());
          	case TCKind._tk_string:
	            return new String  (is.read_string  ());
          	case TCKind._tk_wstring:
	            return new String  (is.read_wstring ());
           	case TCKind._tk_octet:
	            return new Byte    (is.read_octet   ());
            case TCKind._tk_double:
                return new Double  (is.read_double  ());
            case TCKind._tk_float:
                return new Float   (is.read_float   ());
            case TCKind._tk_long:
                return new Integer (is.read_long    ());
            case TCKind._tk_ulong:
            case TCKind._tk_enum :
                return new Integer (is.read_ulong   ());
            case TCKind._tk_longlong:
                return new Long    (is.read_longlong());
            case TCKind._tk_ulonglong:
                return new Long    (is.read_ulonglong());
            case TCKind._tk_short:
                return new Short   (is.read_short   ());
            case TCKind._tk_ushort:
                return new Short   (is.read_ushort  ());
            case TCKind._tk_objref   :
                return new CORBAObject (is.read_Object ());
        }

        throw new org.omg.CORBA.MARSHAL ("cannot marshal type: " + tc, 0, CompletionStatus.COMPLETED_NO);
    }

    public static void copyObject (InputStream is, OutputStream os, TypeCode tc)
        throws org.omg.CORBA.MARSHAL
    {
        switch (tc.kind().value())
        {
            case TCKind._tk_null:
            case TCKind._tk_void:
                break;
            case TCKind._tk_boolean:
                os.write_boolean (is != null ? is.read_boolean () : false);
                break;
            case TCKind._tk_char:
                os.write_char (is != null ? is.read_char () : '\0');
                break;
            case TCKind._tk_wchar:
                os.write_wchar (is != null ? is.read_wchar () : '\0');
                break;
            case TCKind._tk_string:
                os.write_string (is != null ? is.read_string () : "");
                break;
            case TCKind._tk_wstring:
                os.write_wstring (is != null ? is.read_wstring () : "");
                break;
            case TCKind._tk_octet:
                os.write_octet (is != null ? is.read_octet () : 0);
                break;
            case TCKind._tk_double:
                os.write_double (is != null ? is.read_double () : 0);
                break;
            case TCKind._tk_float:
                os.write_float (is != null ? is.read_float () : 0);
                break;
            case TCKind._tk_short:
                os.write_short (is != null ? is.read_short () : 0);
                break;
            case TCKind._tk_ushort:
                os.write_ushort (is != null ? is.read_ushort () : 0);
                break;
            case TCKind._tk_long:
                os.write_long (is != null ? is.read_long () : 0);
                break;
            case TCKind._tk_ulong:
            case TCKind._tk_enum :
                os.write_ulong (is != null ? is.read_ulong () : 0);
                break;
            case TCKind._tk_longlong:
                os.write_longlong (is != null ? is.read_longlong () : 0);
                break;
            case TCKind._tk_ulonglong:
                os.write_ulonglong (is != null ? is.read_ulonglong () : 0);
                break;
            case TCKind._tk_objref   :
                os.write_Object (is != null ? is.read_Object () : null);
                break;
            default:
                throw new org.omg.CORBA.MARSHAL ("cannot marshal type: " + tc, 0, CompletionStatus.COMPLETED_NO);
        }
    }

    public static void writeObject (OutputStream os, TypeCode tc, Object value)
        throws org.omg.CORBA.MARSHAL
    {
        int tkind  = tc.kind().value ();
        boolean ok = false;

        if (value == Simple.class)    
            value = ((Simple)value).getDefaultValue (null);

        switch (tkind)
        {
            case TCKind._tk_null:
            case TCKind._tk_void:
                ok = true;
                break;
            case TCKind._tk_string :
            case TCKind._tk_wstring:
                if (ok = value instanceof String)
                {
                    if (tkind == TCKind._tk_string)
                        os.write_string  ((String)value);
                    else
                        os.write_wstring ((String)value);
                }
                break;
            case TCKind._tk_char :
            case TCKind._tk_wchar:
                char c;

                if (ok = value instanceof Character)
                    c = ((Character)value).charValue ();
                else
                if (ok = value instanceof String)
                    c = ((String)value).charAt (0);
                else
                    break;

                if (tkind == TCKind._tk_char)
                    os.write_char  (c);
                else
                    os.write_wchar (c);
                break;
            case TCKind._tk_octet:
                if (ok = value instanceof  Byte )
                        os.write_octet (( (Byte) value).byteValue ());
                else
                if (ok = value instanceof Number)
                        os.write_octet (((Number)value).byteValue ());
                break;

            case TCKind._tk_boolean:
                if (ok = value instanceof Boolean)
                    os.write_boolean (((Boolean)value).booleanValue ());
                break;

            case TCKind._tk_double:
            case TCKind._tk_float:
            case TCKind._tk_short:
            case TCKind._tk_ushort:
            case TCKind._tk_long:
            case TCKind._tk_ulong:
            case TCKind._tk_longlong:
            case TCKind._tk_ulonglong:
                if (!(ok = value instanceof Number))
                    break;
                switch (tkind)  {
                    case TCKind._tk_double:
                        os.write_double (((Number)value).doubleValue ());
                        break;
                    case TCKind._tk_float:
                        os.write_double (((Number)value).floatValue  ());
                        break;
                    case TCKind._tk_short:
                        os.write_short  (((Number)value).shortValue  ());
                        break;
                    case TCKind._tk_ushort:
                        os.write_ushort (((Number)value).shortValue  ());
                        break;
                    case TCKind._tk_long:
                        os.write_long   (((Number)value).intValue    ());
                        break;
                    case TCKind._tk_ulong:
                        os.write_ulong  (((Number)value).intValue    ());
                        break;
                    case TCKind._tk_longlong:
                        os.write_longlong (((Number)value).longValue ());
                        break;
                    case TCKind._tk_ulonglong:
                        os.write_ulonglong (((Number)value).longValue());
                };
                break;
            case TCKind._tk_enum     :
                if (ok = value instanceof Number)
                    os.write_ulong  (((Number)value).intValue ());
                break;
            case TCKind._tk_objref   :
                if (ok = value instanceof CORBAObject)
                    os.write_Object (((CORBAObject)value).obj);
                break;
        }

        if (!ok)
            throw new org.omg.CORBA.MARSHAL ("cannot cast value: " + value + " to type: " + tc, 0, CompletionStatus.COMPLETED_NO);
    }

    public void decode ()
        throws org.omg.CORBA.MARSHAL
    {
        obj = extractObject (any.create_input_stream (), any.type ());
    }

    public void encode ()
        throws org.omg.CORBA.MARSHAL
    {
        TypeCode tc = any.type ();

        OutputStream os = any.create_output_stream ();

        Marshal.writeObject (os, tc, this);
        any.read_value (os.create_input_stream (), tc);
    }

    public static boolean handledType (int tkind)
    {
        switch (tkind)
        {
            case TCKind._tk_null     :
            case TCKind._tk_void     :
            case TCKind._tk_boolean  :
          	case TCKind._tk_string   :
          	case TCKind._tk_wstring  :
           	case TCKind._tk_octet    :
            case TCKind._tk_double   :
            case TCKind._tk_float    :
            case TCKind._tk_long     :
            case TCKind._tk_ulong    :
            case TCKind._tk_enum     :
            case TCKind._tk_longlong :
            case TCKind._tk_ulonglong:
            case TCKind._tk_short    :
            case TCKind._tk_ushort   :
            case TCKind._tk_objref   :
                return true;
        }
        return false;
    }

    public Any any;
    Object obj;
